@@ -61,8 +61,8 @@ minetest.register_node("testnodes:attached", {
 -- when the node it attaches to is gone.
 minetest.register_node("testnodes:attached_wallmounted", {
 	description = S("Wallmounted Attached Node").."\n"..
-		S("Attaches to wall; drops as item if neighbor node is gone").."\n"..
-		S("param2 = wallmounted rotation (0..5)"),
+		S("Attaches to solid node it was placed on; drops as item if neighbor node is gone").."\n"..
+		S("param2 = wallmounted rotation (0..7)"),
 	paramtype2 = "wallmounted",
 	tiles = {
 		"testnodes_attachedw_top.png",
@@ -72,9 +72,29 @@ minetest.register_node("testnodes:attached_wallmounted", {
 	groups = { attached_node = 1, dig_immediate = 3 },
 })
 
+-- This node attaches to the side of a node and drops as item
+-- when the node it attaches to is gone.
+-- Also adds vertical 90° rotation variants.
+minetest.register_node("testnodes:attached_wallmounted_rot", {
+	description = S("Rotatable Wallmounted Attached Node").."\n"..
+		S("Attaches to solid node it was placed on; drops as item if neighbor node is gone").."\n"..
+		S("param2 = wallmounted rotation (0..7)").."\n"..
+		S("May be rotated by 90° if placed at floor or ceiling"),
+	paramtype2 = "wallmounted",
+	tiles = {
+		"testnodes_attachedwr_top.png",
+		"testnodes_attachedwr_bottom.png",
+		"testnodes_attachedwr_side.png",
+	},
+	wallmounted_rotate_vertical = true,
+	groups = { attached_node = 1, dig_immediate = 3 },
+})
+
 -- Wallmounted node that always attaches to the floor
 minetest.register_node("testnodes:attached_wallmounted_floor", {
-	description = S("Floor-Attached Wallmounted Node"),
+	description = S("Floor-Attached Wallmounted Node").."\n"..
+		S("Drops as item if no solid node below (regardless of rotation)").."\n"..
+		S("param2 = wallmounted rotation (visual only) (0..7)"),
 	paramtype2 = "wallmounted",
 	tiles = {
 		"testnodes_attached_top.png",
@@ -85,10 +105,28 @@ minetest.register_node("testnodes:attached_wallmounted_floor", {
 	color = "#FF8080",
 })
 
+-- Wallmounted node that always attaches to the floor.
+-- Also adds 90° rotation variants.
+minetest.register_node("testnodes:attached_wallmounted_floor_rot", {
+	description = S("Rotatable Floor-Attached Wallmounted Node").."\n"..
+		S("Drops as item if no solid node below (regardless of rotation)").."\n"..
+		S("param2 = wallmounted rotation (visual only) (0..7)").."\n"..
+		S("May be rotated by 90° if placed at floor or ceiling"),
+	paramtype2 = "wallmounted",
+	tiles = {
+		"testnodes_attachedfr_top.png",
+		"testnodes_attachedfr_bottom.png",
+		"testnodes_attachedfr_side.png",
+	},
+	wallmounted_rotate_vertical = true,
+	groups = { attached_node = 3, dig_immediate = 3 },
+})
+
 -- This node attaches to the ceiling and drops as item
 -- when the ceiling is gone.
 minetest.register_node("testnodes:attached_top", {
-	description = S("Ceiling-Attached Node"),
+	description = S("Ceiling-Attached Node").."\n"..
+		S("Drops as item if no solid node above"),
 	tiles = {
 		"testnodes_attached_bottom.png",
 		"testnodes_attached_top.png",
@@ -99,7 +137,9 @@ minetest.register_node("testnodes:attached_top", {
 
 -- Same as wallmounted attached, but for facedir
 minetest.register_node("testnodes:attached_facedir", {
-	description = S("Facedir Attached Node"),
+	description = S("Facedir Attached Node").."\n"..
+		S("Attaches to a neighboring solid node; drops as item if that node is gone").."\n"..
+		S("param2 = facedir rotation (0..23)"),
 	paramtype2 = "facedir",
 	tiles = {
 		"testnodes_attachedf_side.png^[transformR180",
@@ -114,7 +154,9 @@ minetest.register_node("testnodes:attached_facedir", {
 
 -- Same as facedir attached, but for 4dir
 minetest.register_node("testnodes:attached_4dir", {
-	description = S("4dir Attached Node"),
+	description = S("4dir Attached Node").."\n"..
+		S("Attaches to the side of a solid node; drops as item if that node is gone").."\n"..
+		S("param2 = 4dir rotation (0..3)"),
 	paramtype2 = "4dir",
 	tiles = {
 		"testnodes_attached4_side.png^[transformR180",
@@ -145,6 +187,10 @@ minetest.register_node("testnodes:nojump_walkable", {
 	tiles = {"testnodes_nojump_top.png"},
 })
 
+local climbable_nodebox = {
+	type = "regular",
+}
+
 -- Climbable up and down with jump and sneak keys
 minetest.register_node("testnodes:climbable", {
 	description = S("Climbable Node").."\n"..
@@ -156,8 +202,10 @@ minetest.register_node("testnodes:climbable", {
 	paramtype = "light",
 	sunlight_propagates = true,
 	is_ground_content = false,
-	tiles ={"testnodes_climbable_side.png"},
-	drawtype = "glasslike",
+	tiles = {"testnodes_climbable_top.png","testnodes_climbable_top.png","testnodes_climbable_side.png"},
+	use_texture_alpha = "clip",
+	drawtype = "nodebox",
+	node_box = climbable_nodebox,
 	groups = {dig_immediate=3},
 })
 
@@ -169,8 +217,39 @@ minetest.register_node("testnodes:climbable_nojump", {
 	walkable = false,
 
 	groups = {disable_jump=1, dig_immediate=3},
-	drawtype = "glasslike",
-	tiles ={"testnodes_climbable_nojump_side.png"},
+	drawtype = "nodebox",
+	node_box = climbable_nodebox,
+	tiles = {"testnodes_climbable_nojump_top.png","testnodes_climbable_nojump_top.png","testnodes_climbable_nojump_side.png"},
+	use_texture_alpha = "clip",
+	paramtype = "light",
+	sunlight_propagates = true,
+})
+
+
+minetest.register_node("testnodes:climbable_nodescend", {
+	description = S("Upwards-climbable Node"),
+	climbable = true,
+	walkable = false,
+
+	groups = {disable_descend=1, dig_immediate=3},
+	drawtype = "nodebox",
+	node_box = climbable_nodebox,
+	tiles = {"testnodes_climbable_nodescend_top.png","testnodes_climbable_nodescend_top.png","testnodes_climbable_nodescend_side.png"},
+	use_texture_alpha = "clip",
+	paramtype = "light",
+	sunlight_propagates = true,
+})
+
+minetest.register_node("testnodes:climbable_nodescend_nojump", {
+	description = S("Horizontal-only Climbable Node"),
+	climbable = true,
+	walkable = false,
+
+	groups = {disable_jump=1, disable_descend=1, dig_immediate=3},
+	drawtype = "nodebox",
+	node_box = climbable_nodebox,
+	tiles = {"testnodes_climbable_noclimb_top.png","testnodes_climbable_noclimb_top.png","testnodes_climbable_noclimb_side.png"},
+	use_texture_alpha = "clip",
 	paramtype = "light",
 	sunlight_propagates = true,
 })
@@ -198,7 +277,6 @@ minetest.register_node("testnodes:liquid_nojump", {
 	paramtype = "light",
 	pointable = false,
 	liquids_pointable = true,
-	buildable_to = true,
 	is_ground_content = false,
 	post_effect_color = {a = 70, r = 255, g = 0, b = 200},
 })
@@ -228,7 +306,6 @@ minetest.register_node("testnodes:liquidflowing_nojump", {
 	paramtype2 = "flowingliquid",
 	pointable = false,
 	liquids_pointable = true,
-	buildable_to = true,
 	is_ground_content = false,
 	post_effect_color = {a = 70, r = 255, g = 0, b = 200},
 })
@@ -293,7 +370,60 @@ minetest.register_node("testnodes:liquidflowing_noswim", {
 	post_effect_color = {a = 70, r = 255, g = 200, b = 200},
 })
 
+-- A liquid in which you can't actively descend.
+-- Note: You'll still descend slowly by doing nothing.
+minetest.register_node("testnodes:liquid_nodescend", {
+	description = S("No-descending Liquid Source Node"),
+	liquidtype = "source",
+	liquid_range = 0,
+	liquid_viscosity = 0,
+	liquid_alternative_flowing = "testnodes:liquidflowing_nodescend",
+	liquid_alternative_source = "testnodes:liquid_nodescend",
+	liquid_renewable = false,
+	groups = {disable_descend=1, dig_immediate=3},
+	walkable = false,
 
+	drawtype = "liquid",
+	tiles = {"testnodes_liquidsource.png^[colorize:#FFFF00:127"},
+	special_tiles = {
+		{name = "testnodes_liquidsource.png^[colorize:#FFFF00:127", backface_culling = false},
+		{name = "testnodes_liquidsource.png^[colorize:#FFFF00:127", backface_culling = true},
+	},
+	use_texture_alpha = "blend",
+	paramtype = "light",
+	pointable = false,
+	liquids_pointable = true,
+	is_ground_content = false,
+	post_effect_color = {a = 70, r = 255, g = 255, b = 200},
+})
+
+-- A liquid in which you can't actively descend (flowing variant)
+minetest.register_node("testnodes:liquidflowing_nodescend", {
+	description = S("No-descending Flowing Liquid Node"),
+	liquidtype = "flowing",
+	liquid_range = 1,
+	liquid_viscosity = 0,
+	liquid_alternative_flowing = "testnodes:liquidflowing_nodescend",
+	liquid_alternative_source = "testnodes:liquid_nodescend",
+	liquid_renewable = false,
+	groups = {disable_descend=1, dig_immediate=3},
+	walkable = false,
+
+
+	drawtype = "flowingliquid",
+	tiles = {"testnodes_liquidflowing.png^[colorize:#FFFF00:127"},
+	special_tiles = {
+		{name = "testnodes_liquidflowing.png^[colorize:#FFFF00:127", backface_culling = false},
+		{name = "testnodes_liquidflowing.png^[colorize:#FFFF00:127", backface_culling = false},
+	},
+	use_texture_alpha = "blend",
+	paramtype = "light",
+	paramtype2 = "flowingliquid",
+	pointable = false,
+	liquids_pointable = true,
+	is_ground_content = false,
+	post_effect_color = {a = 70, r = 255, g = 255, b = 200},
+})
 
 -- Nodes that modify fall damage (various damage modifiers)
 for i=-100, 100, 25 do
@@ -340,17 +470,17 @@ local MAX_BOUNCE_NONJUMPY = 140
 for i=-MAX_BOUNCE_NONJUMPY, MAX_BOUNCE_JUMPY, 20 do
 	if i ~= 0 then
 		local desc
-		local val = math.floor(((i-20)/200)*255)
-		local val2 = math.max(0, 200 - val)
+		local val = math.floor((math.abs(i) - 20) / 200 * 255)
+		local val2 = math.max(0, 255 - val)
 		local num = string.format("%03d", math.abs(i))
 		if i > 0 then
 			desc = S("Bouncy Node (@1%), jumpy", i).."\n"..
 				S("Sneaking/jumping affects bounce")
-			color = { r=255-val, g=255-val, b=255, a=255 }
+			color = { r=val2, g=val2, b=255, a=255 }
 		else
 			desc = S("Bouncy Node (@1%), non-jumpy", math.abs(i)).."\n"..
 				S("Sneaking/jumping does not affect bounce")
-			color = { r=val, g=255, b=val, a=255 }
+			color = { r=val2, g=255, b=val2, a=255 }
 			num = "NEG"..num
 		end
 		minetest.register_node("testnodes:bouncy"..num, {
@@ -430,10 +560,11 @@ minetest.register_node("testnodes:climbable_move_resistance_4", {
 	climbable = true,
 	move_resistance = 4,
 
-	drawtype = "glasslike",
+	drawtype = "nodebox",
 	paramtype = "light",
 	sunlight_propagates = true,
-	tiles = {"testnodes_climbable_resistance_side.png"},
+	tiles = {"testnodes_climbable_top.png","testnodes_climbable_top.png","testnodes_climbable_resistance_side.png"},
+	use_texture_alpha = "clip",
 	is_ground_content = false,
 	groups = { dig_immediate = 3 },
 })
@@ -499,3 +630,56 @@ minetest.register_node("testnodes:drowning_1", {
 	groups = {dig_immediate=3},
 })
 
+-- post_effect_color_shaded
+
+minetest.register_node("testnodes:post_effect_color_shaded_false", {
+	description = S("\"post_effect_color_shaded = false\" Node"),
+
+	drawtype = "allfaces",
+	tiles = {"testnodes_post_effect_color_shaded_false.png"},
+	use_texture_alpha = "blend",
+	paramtype = "light",
+	sunlight_propagates = true,
+	post_effect_color = {a = 128, r = 255, g = 255, b = 255},
+	post_effect_color_shaded = false,
+
+	walkable = false,
+	is_ground_content = false,
+	groups = {dig_immediate=3},
+})
+
+minetest.register_node("testnodes:post_effect_color_shaded_true", {
+	description = S("\"post_effect_color_shaded = true\" Node"),
+
+	drawtype = "allfaces",
+	tiles = {"testnodes_post_effect_color_shaded_true.png"},
+	use_texture_alpha = "blend",
+	paramtype = "light",
+	sunlight_propagates = true,
+	post_effect_color = {a = 128, r = 255, g = 255, b = 255},
+	post_effect_color_shaded = true,
+
+	walkable = false,
+	is_ground_content = false,
+	groups = {dig_immediate=3},
+})
+
+-- Pointability
+
+-- Register wrapper for compactness
+local function register_pointable_test_node(name, description, pointable)
+	local texture = "testnodes_"..name..".png"
+	minetest.register_node("testnodes:"..name, {
+		description = S(description),
+		tiles = {texture},
+		drawtype = "glasslike_framed",
+		paramtype = "light",
+		walkable = false,
+		pointable = pointable,
+		groups = {dig_immediate=3, [name.."_test"]=1},
+	})
+end
+
+register_pointable_test_node("pointable", "Pointable Node", true)
+register_pointable_test_node("not_pointable", "Not Pointable Node", false)
+register_pointable_test_node("blocking_pointable", "Blocking Pointable Node", "blocking")
